@@ -1,12 +1,13 @@
 from biblemate.core.systems import *
 from biblemate.ui.prompts import getInput
+from biblemate.ui.text_area import getTextArea
 from biblemate.ui.info import get_banner
 from biblemate.ui.selection_dialog import TerminalModeDialogs
 from biblemate import config, AGENTMAKE_CONFIG, BIBLEMATEDATA, fix_string, write_user_config
 from biblemate.uba.api import run_uba_api, DEFAULT_MODULES
 from pathlib import Path
 import urllib.parse
-import asyncio, re, os, subprocess, click, gdown, pprint, argparse, json, zipfile, shutil
+import asyncio, re, os, subprocess, click, gdown, pprint, argparse, json, zipfile
 from copy import deepcopy
 from alive_progress import alive_bar
 from fastmcp import Client
@@ -158,6 +159,7 @@ Get a static text-based response directly from a text-based AI model without usi
     return tools, tools_schema, master_available_tools, available_tools, tool_descriptions, prompts, prompts_schema, resources, templates
 
 def edit_temp_file(initial_content: str) -> str:
+    config.current_prompt = ""
     temp_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), "temp", "edit.md")
     writeTextFile(temp_file, initial_content)
     edit_file(temp_file)
@@ -242,7 +244,7 @@ async def main_async():
         master_plan = ""
         messages = deepcopy(DEFAULT_MESSAGES) # set the tone
 
-        while not user_request == ".quit":
+        while not user_request == ".exit":
 
             # spinner while thinking
             async def thinking(process, description=None):
@@ -284,8 +286,16 @@ async def main_async():
             if not len(messages) == len(DEFAULT_MESSAGES):
                 console.rule()
             elif APP_START:
-                print()
                 APP_START = False
+                print()
+                # check for updates
+                latest_version = getPackageLatestVersion("biblemate")
+                current_version = readTextFile(os.path.join(os.path.dirname(os.path.realpath(__file__)), "version.txt")).strip()
+                if latest_version and str(latest_version).strip() != current_version:
+                    console.rule()
+                    console.print(Markdown(f"## A new version of BibleMate AI is available: {latest_version} (you are using {current_version}).\n\nTo upgrade, close `BibleMate AI` first and run `pip install --upgrade biblemate`."))
+                    console.rule()
+                # check connection
                 try:
                     agentmake("Hello!", system=DEFAULT_SYSTEM)
                 except Exception as e:
@@ -297,54 +307,40 @@ async def main_async():
                         console.print("Restart to make the changes in the backend effective!", justify="center")
                         console.rule()
                         exit()
-                # check for updates
-                latest_version = getPackageLatestVersion("biblemate")
-                current_version = readTextFile(os.path.join(os.path.dirname(os.path.realpath(__file__)), "version.txt")).strip()
-                if latest_version and latest_version.strip() != current_version:
-                    console.rule()
-                    if USER_OS == "Windows":
-                        console.print(Markdown(f"## A new version of BibleMate AI is available: {latest_version} (you are using {current_version}).\n\nPlease run `pip install --upgrade biblemate` to upgrade!"))
-                        console.rule()
-                    elif click.confirm(f"A new version of BibleMate AI is available: {latest_version} (you are using {current_version}). Do you want to upgrade now?", default=True) and shutil.which("pip"):
-                        pip_path = shutil.which("pip")
-                        if os.system(f"{pip_path} install --upgrade biblemate") == 0:
-                            console.print("Restart to make the changes effective!", justify="center")
-                            console.rule()
-                            exit()
-                        else:
-                            console.print(Markdown(f"## A new version of BibleMate AI is available: {latest_version} (you are using {current_version}).\n\nPlease run `pip install --upgrade biblemate` to upgrade!"))
-                            console.rule()
             # Original user request
             # note: `python3 -m rich.emoji` for checking emoji
             console.print("Enter your request :smiley: :" if len(messages) == len(DEFAULT_MESSAGES) else "Enter a follow-up request :flexed_biceps: :")
             action_list = {
                 ".new": "new conversation",
-                ".quit": "quit",
-                ".backend": "change backend",
-                ".mode": "change AI mode",
+                ".exit": "quit",
+                ".backend": "configure backend",
+                ".mode": "configure AI mode",
+                ".steps": "configure the maximum number of steps allowed",
+                ".matches": "configure the maximum number of semantic matches",
                 ".tools": "list available tools",
                 ".plans": "list available plans",
                 ".resources": "list UniqueBible resources",
                 ".promptengineer": "toggle auto prompt engineering",
                 ".lite": "toggle lite context",
-                ".steps": "configure the maximum number of steps",
-                ".matches": "configure the maximum number of semantic matches",
                 ".edit": "load the current conversation",
                 ".backup": "backup conversation",
                 ".load": "load a saved conversation",
                 ".open": "open a file or directory",
                 ".download": "download data files",
+                ".ideas": "generate ideas for prompts to try",
                 ".help": "help page",
             }
-            input_suggestions = list(action_list.keys())+["@ ", "@@ "]+[f"@{t} " for t in available_tools]+[f"{p} " for p in prompt_list]+[f"//{r}" for r in resources.keys()]+template_list+resource_suggestions # "" is for generating ideas
+            input_suggestions = list(action_list.keys())+["@ ", "@@ "]+[f"@{t} " for t in available_tools]+[f"{p} " for p in prompt_list]+[f"//{r}" for r in resources.keys()]+template_list+resource_suggestions
             if args.default:
                 user_request = " ".join(args.default)
                 args.default = None # reset to avoid repeated use
             else:
-                user_request = await getInput("> ", input_suggestions)
-            if user_request == ".editprompt": # edit current prompt in editor
+                user_request = await getTextArea(input_suggestions=input_suggestions)
+            if not user_request.strip():
+                continue
+            elif user_request == ".editprompt": # edit current prompt in editor
                 user_request = edit_temp_file(config.current_prompt)
-            while not user_request.strip():
+            elif user_request == ".ideas":
                 # Generate ideas for `prompts to try`
                 ideas = ""
                 async def generate_ideas():
@@ -358,7 +354,7 @@ async def main_async():
                 console.print(Markdown(f"## Ideas\n\n{ideas}\n\n"))
                 console.rule()
                 # Get input again
-                user_request = await getInput("> ", input_suggestions)
+                user_request = await getTextArea(input_suggestions=input_suggestions)
                 if user_request == ".editprompt": # edit current prompt in editor
                     user_request = edit_temp_file(config.current_prompt)
 
@@ -473,8 +469,29 @@ async def main_async():
                 if user_request == ".backup":
                     backup_conversation(messages, master_plan, console)
                 elif user_request == ".help":
+                    actions = "\n".join([f"- `{k}`: {v}" for k, v in action_list.items()])
+                    help_info = f"""## Key Commands
+
+{actions}
+
+## Key Bindings
+
+- `Ctrl+K`: help info
+- `Ctrl+E` or `Ctrl+P`: edit current prompt
+- `Ctrl+Q`: exit current prompt
+- `Ctrl+R`: reset current prompt
+- `Ctrl+S` or `Esc+ENTER` or `Alt+ENTER`: submit current prompt
+- `Ctrl+Z`: undo current prompt
+- `Ctrl+N`: new conversation
+- `Shift+TAB`: insert four spaces
+- `TAB` or `Ctrl+I`: open input suggestion menu
+- `Esc`: close input suggestion menu
+
+## More
+                    
+Viist https://github.com/eliranwong/biblemate"""
                     console.rule()
-                    console.print(Markdown("Viist https://github.com/eliranwong/biblemate for help."))
+                    console.print(Markdown(help_info))
                     console.rule()
                 elif user_request == ".tools":
                     enabled_tools = await dialogs.getMultipleSelection(
@@ -532,7 +549,7 @@ async def main_async():
                 elif user_request == ".steps":
                     console.rule()
                     console.print("Enter below the maximum number of steps allowed:")
-                    max_steps = await getInput("> ", number_validator=True, default_entry=str(config.max_steps))
+                    max_steps = await getInput(number_validator=True, default_entry=str(config.max_steps))
                     if max_steps:
                         config.max_steps = int(max_steps)
                         write_user_config()
@@ -541,7 +558,7 @@ async def main_async():
                 elif user_request == ".matches":
                     console.rule()
                     console.print("Enter below the maximum number of semantic matches allowed:")
-                    max_semantic_matches = await getInput("> ", number_validator=True, default_entry=str(config.max_semantic_matches))
+                    max_semantic_matches = await getInput(number_validator=True, default_entry=str(config.max_semantic_matches))
                     if max_semantic_matches:
                         config.max_semantic_matches = int(max_semantic_matches)
                         write_user_config()
@@ -603,7 +620,7 @@ async def main_async():
                         console.rule()
                         console.print(f"`{ai_mode.capitalize()}` Mode Enabled", justify="center")
                         console.rule()
-                elif user_request in (".new", ".quit"):
+                elif user_request in (".new", ".exit"):
                     backup_conversation(messages, master_plan, console) # backup
                 # reset
                 if user_request == ".new":
@@ -778,7 +795,7 @@ Available tools are: {available_tools}.
                         master_plan_edit = await getInput(default_entry=master_plan)
                         if master_plan_edit == ".editprompt": # edit current prompt in editor
                             master_plan_edit = edit_temp_file(config.current_prompt)
-                        if not master_plan_edit or master_plan_edit == ".quit":
+                        if not master_plan_edit or master_plan_edit == ".exit":
                             if messages and messages[-1].get("role", "") == "user":
                                 messages = messages[:-1]
                             console.rule()
@@ -860,7 +877,7 @@ Available tools are: {available_tools}.
                     next_step_edit = await getInput(default_entry=next_step)
                     if next_step_edit == ".editprompt": # edit current prompt in editor
                         next_step_edit = edit_temp_file(config.current_prompt)
-                    if not next_step_edit or next_step_edit == ".quit":
+                    if not next_step_edit or next_step_edit == ".exit":
                         console.rule()
                         console.print("I've stopped processing for you.")
                         break
